"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _promise = _interopRequireDefault(require("@babel/runtime/core-js/promise"));

var _isObservable = _interopRequireDefault(require("is-observable"));

var _isPromise = _interopRequireDefault(require("is-promise"));

var _observableToPromise = _interopRequireDefault(require("observable-to-promise"));

var _onetime = _interopRequireDefault(require("onetime"));

// pipe the promise's result/error to the callback
function pipePromise(promise, done) {
  var onFulfilled = function onFulfilled(result) {
    done(null, result);
  };

  var onRejected = function onRejected(error) {
    // ensure a promise rejected with a falsey value isn't
    // interpreted as a success:
    // https://github.com/gulpjs/async-done/issues/42
    done(error || new Error('Promise rejected without Error'));
  };

  promise.then(onFulfilled, onRejected);
}

function asyncAny(task, callback) {
  var promise;

  if (!callback) {
    promise = new _promise.default(function (resolve, reject) {
      callback = function callback(error, result) {
        if (error) {
          reject(error);
        } else {
          resolve(result);
        }
      };
    });
  }

  var done = (0, _onetime.default)(callback);
  var result = task(done); // prefer promise if the result is both a promise and an observable:
  // https://github.com/gulpjs/async-done/issues/36
  // https://github.com/avajs/ava/blob/v1.0.0-beta.4/lib/assert.js#L345-L351
  //
  // XXX isObservable is currently buggy with RxJS@6 observables
  // https://github.com/sindresorhus/is-observable/issues/1#issuecomment-387843191

  if ((0, _isObservable.default)(result) && !(0, _isPromise.default)(result)) {
    result = (0, _observableToPromise.default)(result);
  }

  if ((0, _isPromise.default)(result)) {
    pipePromise(result, done);
  }

  return promise;
}

module.exports = asyncAny;